@@ -3,7 +3,7 @@
 """
 The MIT License (MIT)
 
-Copyright (c) 2015-2019 Rapptz
+Copyright (c) 2015-2020 Rapptz
 
 Permission is hereby granted, free of charge, to any person obtaining a
 copy of this software and associated documentation files (the "Software"),
@@ -41,128 +41,15 @@ from .enums import VoiceRegion, Status, ChannelType, try_enum, VerificationLevel
 from .mixins import Hashable
 from .user import User
 from .invite import Invite
-from .iterators import AuditLogIterator
+from .iterators import AuditLogIterator, MemberIterator
 from .webhook import Webhook
 from .widget import Widget
 from .asset import Asset
+from .flags import SystemChannelFlags
 
 BanEntry = namedtuple('BanEntry', 'reason user')
 _GuildLimit = namedtuple('_GuildLimit', 'emoji bitrate filesize')
 
-class _flag_descriptor:
-    def __init__(self, func):
-        self.flag = func(None)
-        self.__doc__ = func.__doc__
-
-    def __get__(self, instance, owner):
-        return instance._has_flag(self.flag)
-
-    def __set__(self, instance, value):
-        instance._set_flag(self.flag, value)
-
-def fill_with_flags(cls):
-    cls.VALID_FLAGS = {
-        name: value.flag
-        for name, value in cls.__dict__.items()
-        if isinstance(value, _flag_descriptor)
-    }
-
-    max_bits = max(cls.VALID_FLAGS.values()).bit_length()
-    cls.ALL_OFF_VALUE = -1 + (2 ** max_bits)
-    return cls
-
-@fill_with_flags
-class SystemChannelFlags:
-    r"""Wraps up a Discord system channel flag value.
-
-    Similar to :class:`Permissions`\, the properties provided are two way.
-    You can set and retrieve individual bits using the properties as if they
-    were regular bools. This allows you to edit the system flags easily.
-
-    To construct an object you can pass keyword arguments denoting the flags
-    to enable or disable.
-
-    .. container:: operations
-
-        .. describe:: x == y
-
-            Checks if two flags are equal.
-        .. describe:: x != y
-
-            Checks if two flags are not equal.
-        .. describe:: hash(x)
-
-               Return the flag's hash.
-        .. describe:: iter(x)
-
-               Returns an iterator of ``(name, value)`` pairs. This allows it
-               to be, for example, constructed as a dict or a list of pairs.
-
-    Attributes
-    -----------
-    value: :class:`int`
-        The raw value. This value is a bit array field of a 53-bit integer
-        representing the currently available flags. You should query
-        flags via the properties rather than using this raw value.
-    """
-    __slots__ = ('value',)
-
-    def __init__(self, **kwargs):
-        self.value = self.ALL_OFF_VALUE
-        for key, value in kwargs.items():
-            if key not in self.VALID_FLAGS:
-                raise TypeError('%r is not a valid flag name.' % key)
-            setattr(self, key, value)
-
-    @classmethod
-    def _from_value(cls, value):
-        self = cls.__new__(cls)
-        self.value = value
-        return self
-
-    def __eq__(self, other):
-        return isinstance(other, SystemChannelFlags) and self.value == other.value
-
-    def __ne__(self, other):
-        return not self.__eq__(other)
-
-    def __hash__(self):
-        return hash(self.value)
-
-    def __repr__(self):
-        return '<SystemChannelFlags value=%s>' % self.value
-
-    def __iter__(self):
-        for name, value in self.__class__.__dict__.items():
-            if isinstance(value, _flag_descriptor):
-                yield (name, self._has_flag(value.flag))
-
-    # For some reason the flags in the Discord API are "inverted"
-    # ergo, if they're set then it means "suppress" (off in the GUI toggle)
-    # Since this is counter-intuitive from an API perspective and annoying
-    # these will be inverted automatically
-
-    def _has_flag(self, o):
-        return (self.value & o) != o
-
-    def _set_flag(self, o, toggle):
-        if toggle is True:
-            self.value &= ~o
-        elif toggle is False:
-            self.value |= o
-        else:
-            raise TypeError('Value to set for SystemChannelFlags must be a bool.')
-
-    @_flag_descriptor
-    def join_notifications(self):
-        """:class:`bool`: Returns True if the system channel is used for member join notifications."""
-        return 1
-
-    @_flag_descriptor
-    def premium_subscriptions(self):
-        """:class:`bool`: Returns True if the system channel is used for Nitro boosting notifications."""
-        return 2
-
 
 class Guild(Hashable):
     """Represents a Discord guild.
@@ -241,10 +128,11 @@ class Guild(Hashable):
         - ``MORE_EMOJI``: Guild is allowed to have more than 50 custom emoji.
         - ``DISCOVERABLE``: Guild shows up in Server Discovery.
         - ``COMMERCE``: Guild can sell things using store channels.
-        - ``LURKABLE``: Users can lurk in this guild via Server Discovery.
+        - ``PUBLIC``: Users can lurk in this guild via Server Discovery.
         - ``NEWS``: Guild can create news channels.
         - ``BANNER``: Guild can upload and use a banner (i.e. :meth:`banner_url`).
         - ``ANIMATED_ICON``: Guild can upload an animated icon.
+        - ``PUBLIC_DISABLED``: Guild cannot be public.
 
     splash: Optional[:class:`str`]
         The guild's invite splash.
@@ -253,16 +141,24 @@ class Guild(Hashable):
         The number goes from 0 to 3 inclusive.
     premium_subscription_count: :class:`int`
         How many users have currently "boosted" this guild.
+    preferred_locale: Optional[:class:`str`]
+        The preferred locale for the guild. Used when filtering Server Discovery
+        results to a specific language.
+    discovery_splash: :class:`str`
+        The guild's discovery splash.
+
+        .. versionadded:: 1.3
     """
 
     __slots__ = ('afk_timeout', 'afk_channel', '_members', '_channels', 'icon',
                  'name', 'id', 'unavailable', 'banner', 'region', '_state',
-                 '_default_role', '_roles', '_member_count', '_large',
+                 '_roles', '_member_count', '_large',
                  'owner_id', 'mfa_level', 'emojis', 'features',
                  'verification_level', 'explicit_content_filter', 'splash',
                  '_voice_states', '_system_channel_id', 'default_notifications',
                  'description', 'max_presences', 'max_members', 'premium_tier',
-                 'premium_subscription_count', '_system_channel_flags')
+                 'premium_subscription_count', '_system_channel_flags',
+                 'preferred_locale', 'discovery_splash', '_rules_channel_id')
 
     _PREMIUM_GUILD_LIMITS = {
         None: _GuildLimit(emoji=50, bitrate=96e3, filesize=8388608),
@@ -383,6 +279,9 @@ class Guild(Hashable):
         self.premium_tier = guild.get('premium_tier', 0)
         self.premium_subscription_count = guild.get('premium_subscription_count') or 0
         self._system_channel_flags = guild.get('system_channel_flags', 0)
+        self.preferred_locale = guild.get('preferred_locale')
+        self.discovery_splash = guild.get('discovery_splash')
+        self._rules_channel_id = utils._get_as_snowflake(guild, 'rules_channel_id')
 
         for mdata in guild.get('members', []):
             member = Member(data=mdata, guild=self, state=state)
@@ -517,7 +416,18 @@ class Guild(Hashable):
         return as_list
 
     def get_channel(self, channel_id):
-        """Returns a :class:`abc.GuildChannel` with the given ID. If not found, returns None."""
+        """Returns a channel with the given ID.
+
+        Parameters
+        -----------
+        channel_id: :class:`int`
+            The ID to search for.
+
+        Returns
+        --------
+        Optional[:class:`.abc.GuildChannel`]
+            The returned channel or ``None`` if not found.
+        """
         return self._channels.get(channel_id)
 
     @property
@@ -534,6 +444,18 @@ class Guild(Hashable):
         """:class:`SystemChannelFlags`: Returns the guild's system channel settings."""
         return SystemChannelFlags._from_value(self._system_channel_flags)
 
+    @property
+    def rules_channel(self):
+        """Optional[:class:`TextChannel`]: Return's the guild's channel used for the rules.
+        Must be a discoverable guild.
+
+        If no channel is set, then this returns ``None``.
+
+        .. versionadded:: 1.3
+        """
+        channel_id = self._rules_channel_id
+        return channel_id and self._channels.get(channel_id)
+
     @property
     def emoji_limit(self):
         """:class:`int`: The maximum number of emoji slots this guild has."""
@@ -557,7 +479,18 @@ class Guild(Hashable):
         return list(self._members.values())
 
     def get_member(self, user_id):
-        """Returns a :class:`Member` with the given ID. If not found, returns None."""
+        """Returns a member with the given ID.
+
+        Parameters
+        -----------
+        user_id: :class:`int`
+            The ID to search for.
+
+        Returns
+        --------
+        Optional[:class:`Member`]
+            The member or ``None`` if not found.
+        """
         return self._members.get(user_id)
 
     @property
@@ -575,13 +508,24 @@ class Guild(Hashable):
         return sorted(self._roles.values())
 
     def get_role(self, role_id):
-        """Returns a :class:`Role` with the given ID. If not found, returns None."""
+        """Returns a role with the given ID.
+
+        Parameters
+        -----------
+        role_id: :class:`int`
+            The ID to search for.
+
+        Returns
+        --------
+        Optional[:class:`Role`]
+            The role or ``None`` if not found.
+        """
         return self._roles.get(role_id)
 
-    @utils.cached_slot_property('_default_role')
+    @property
     def default_role(self):
         """Gets the @everyone role that all members have by default."""
-        return utils.find(lambda r: r.is_default(), self._roles.values())
+        return self.get_role(self.id)
 
     @property
     def owner(self):
@@ -688,6 +632,41 @@ class Guild(Hashable):
         """
         return Asset._from_guild_image(self._state, self.id, self.splash, 'splashes', format=format, size=size)
 
+    @property
+    def discovery_splash_url(self):
+        """:class:`Asset`: Returns the guild's discovery splash asset.
+
+        .. versionadded:: 1.3
+        """
+        return self.discovery_splash_url_as()
+
+    def discovery_splash_url_as(self, *, format='webp', size=2048):
+        """Returns an :class:`Asset` for the guild's discovery splash.
+
+        The format must be one of 'webp', 'jpeg', 'jpg', or 'png'. The
+        size must be a power of 2 between 16 and 4096.
+
+        .. versionadded:: 1.3
+
+        Parameters
+        -----------
+        format: :class:`str`
+            The format to attempt to convert the splash to.
+        size: :class:`int`
+            The size of the image to display.
+
+        Raises
+        ------
+        InvalidArgument
+            Bad image format passed to ``format`` or invalid ``size``.
+
+        Returns
+        --------
+        :class:`Asset`
+            The resulting CDN asset.
+        """
+        return Asset._from_guild_image(self._state, self.id, self.discovery_splash, 'discovery-splashes', format=format, size=size)
+
     @property
     def member_count(self):
         """Returns the true member count regardless of it being loaded fully or not."""
@@ -974,7 +953,8 @@ class Guild(Hashable):
             The new description of the guild. This is only available to guilds that
             contain `VERIFIED` in :attr:`Guild.features`.
         icon: :class:`bytes`
-            A :term:`py:bytes-like object` representing the icon. Only PNG/JPEG supported.
+            A :term:`py:bytes-like object` representing the icon. Only PNG/JPEG supported
+            and GIF for guilds with ``ANIMATED_ICON`` feature.
             Could be ``None`` to denote removal of the icon.
         banner: :class:`bytes`
             A :term:`py:bytes-like object` representing the banner.
@@ -1090,7 +1070,7 @@ class Guild(Hashable):
                 fields['system_channel_id'] = system_channel.id
 
         if 'owner' in fields:
-            if self.owner != self.me:
+            if self.owner_id != self._state.self_id:
                 raise InvalidArgument('To transfer ownership you must be the owner of the guild.')
 
             fields['owner_id'] = fields['owner'].id
@@ -1126,7 +1106,7 @@ class Guild(Hashable):
 
             This method is an API call. For general usage, consider :attr:`channels` instead.
 
-        .. versionadded:: 1.2.0
+        .. versionadded:: 1.2
 
         Raises
         -------
@@ -1152,6 +1132,53 @@ class Guild(Hashable):
 
         return [convert(d) for d in data]
 
+    def fetch_members(self, *, limit=1000, after=None):
+        """|coro|
+
+        Retrieves an :class:`.AsyncIterator` that enables receiving the guild's members.
+
+        .. note::
+
+            This method is an API call. For general usage, consider :attr:`members` instead.
+
+        .. versionadded:: 1.3
+
+        All parameters are optional.
+
+        Parameters
+        ----------
+        limit: Optional[:class:`int`]
+            The number of members to retrieve. Defaults to 1000.
+            Pass ``None`` to fetch all members. Note that this is potentially slow.
+        after: Optional[Union[:class:`.abc.Snowflake`, :class:`datetime.datetime`]]
+            Retrieve members after this date or object.
+            If a date is provided it must be a timezone-naive datetime representing UTC time.
+
+        Raises
+        ------
+        HTTPException
+            Getting the members failed.
+
+        Yields
+        ------
+        :class:`.Member`
+            The member with the member data parsed.
+
+        Examples
+        --------
+
+        Usage ::
+
+            async for member in guild.fetch_members(limit=150):
+                print(member.name)
+
+        Flattening into a list ::
+
+            members = await guild.fetch_members(limit=150).flatten()
+            # members is now a list of Member...
+        """
+        return MemberIterator(self, limit=limit, after=after)
+
     async def fetch_member(self, member_id):
         """|coro|
 
@@ -1474,6 +1501,30 @@ class Guild(Hashable):
         data = await self._state.http.create_custom_emoji(self.id, name, img, roles=roles, reason=reason)
         return self._state.store_emoji(self, data)
 
+    async def fetch_roles(self):
+        """|coro|
+
+        Retrieves all :class:`Role` that the guild has.
+
+        .. note::
+
+            This method is an API call. For general usage, consider :attr:`roles` instead.
+
+        .. versionadded:: 1.3
+
+        Raises
+        -------
+        HTTPException
+            Retrieving the roles failed.
+
+        Returns
+        -------
+        List[:class:`Role`]
+            All roles in the guild.
+        """
+        data = await self._state.http.get_roles(self.id)
+        return [Role(guild=self, state=self._state, data=d) for d in data]
+
     async def create_role(self, *, reason=None, **fields):
         """|coro|
 
@@ -1628,8 +1679,7 @@ class Guild(Hashable):
 
         Returns the guild's special vanity invite.
 
-        The guild must be partnered, i.e. have 'VANITY_URL' in
-        :attr:`~Guild.features`.
+        The guild must have 'VANITY_URL' in :attr:`~Guild.features`.
 
         You must have the :attr:`~Permissions.manage_guild` permission to use
         this as well.
@@ -1768,3 +1818,43 @@ class Guild(Hashable):
         data = await self._state.http.get_widget(self.id)
 
         return Widget(state=self._state, data=data)
+
+    async def query_members(self, query, *, limit=5, cache=True):
+        """|coro|
+
+        Request members that belong to this guild whose username starts with
+        the query given.
+
+        This is a websocket operation and can be slow.
+
+        .. warning::
+
+            Most bots do not need to use this. It's mainly a helper
+            for bots who have disabled ``guild_subscriptions``.
+
+        .. versionadded:: 1.3
+
+        Parameters
+        -----------
+        query: :class:`str`
+            The string that the username's start with. An empty string
+            requests all members.
+        limit: :class:`int`
+            The maximum number of members to send back. This must be
+            a number between 1 and 1000.
+        cache: :class:`bool`
+            Whether to cache the members internally. This makes operations
+            such as :meth:`get_member` work for those that matched.
+
+        Raises
+        -------
+        asyncio.TimeoutError
+            The query timed out waiting for the members.
+
+        Returns
+        --------
+        List[:class:`Member`]
+            The list of members that have matched the query.
+        """
+        limit = limit or 5
+        return await self._state.query_members(self, query=query, limit=limit, cache=cache)
